@@ -1,9 +1,15 @@
 package Test::Differences;
 
+=encoding utf8
+
 =head1 NAME
 
 Test::Differences - Test strings and data structures and show differences if not ok
 
+=head1 VERSION
+
+0.61_01
+
 =head1 SYNOPSIS
 
    use Test;    ## Or use Test::More
@@ -20,7 +26,7 @@ Test::Differences - Test strings and data structures and show differences if not
    use DBI;
 
    ... open connection & prepare statement and @expected_... here...
-   
+
    eq_or_diff $sth->fetchall_arrayref, \@expected_arrays  "testing DBI arrays";
    eq_or_diff $sth->fetchall_hashref,  \@expected_hashes, "testing DBI hashes";
 
@@ -139,8 +145,13 @@ YMMV.
 
 =head1 OPTIONS
 
-There is currently only one option: "context".  This allows you to
-control the amount of context shown:
+The options to C<eq_or_diff> give some fine-grained control over the output.
+
+=over 4
+
+=item * C<context>
+
+This allows you to control the amount of context shown:
 
    eq_or_diff $got, $expected, $name, { context => 50000 };
 
@@ -148,6 +159,23 @@ will show you lots and lots of context.  Normally, eq_or_diff() uses
 some heuristics to determine whether to show 3 lines of context (like
 a normal unified diff) or 25 lines.
 
+=item * C<data_type>
+
+C<text> or C<data>. See C<eq_or_diff_text> and C<eq_or_diff_data> to
+understand this. You can usually ignore this.
+
+=item * C<Sortkeys>
+
+If passed, whatever value is added is used as the argument for L<Data::Dumper>
+Sortkeys option. See the L<Data::Dumper> docs to understand how you can
+control the Sortkeys behavior.
+
+=item * C<filename_a> and C<filename_b>
+
+The column headers to use in the output. They default to 'Got' and 'Expected'.
+
+=back
+
 =head1 DIFF STYLES
 
 For extremely long strings, a table diff can wrap on your screen and be hard
@@ -171,9 +199,9 @@ You can run the following to understand the different diff output styles:
 
  use Test::More 'no_plan';
  use Test::Differences;
- 
+
  my $long_string = join '' => 1..40;
- 
+
  TODO: {
      local $TODO = 'Testing diff styles';
 
@@ -192,7 +220,71 @@ You can run the following to understand the different diff output styles:
      eq_or_diff $long_string, "-$long_string", 'oldstyle diff';
  }
 
-=head1 DEPLOYING 
+=head1 UNICODE
+
+Generally you'll find that the following test output is disappointing.
+
+    use Test::Differences;
+
+    my $want = { 'Traditional Chinese' => '中國' };
+    my $have = { 'Traditional Chinese' => '中国' };
+
+    eq_or_diff $have, $want, 'Unicode, baby';
+
+The output looks like this:
+
+    #   Failed test 'Unicode, baby'
+    #   at t/unicode.t line 12.
+    # +----+----------------------------+----------------------------+
+    # | Elt|Got                         |Expected                    |
+    # +----+----------------------------+----------------------------+
+    # |   0|'Traditional Chinese'       |'Traditional Chinese'       |
+    # *   1|'\xe4\xb8\xad\xe5\x9b\xbd'  |'\xe4\xb8\xad\xe5\x9c\x8b'  *
+    # +----+----------------------------+----------------------------+
+    # Looks like you failed 1 test of 1.
+    Dubious, test returned 1 (wstat 256, 0x100)
+
+This is generally not helpful and someone points out that you didn't declare
+your test program as being utf8, so you do that:
+
+    use Test::Differences;
+    use utf8;
+
+    my $want = { 'Traditional Chinese' => '中國' };
+    my $have = { 'Traditional Chinese' => '中国' };
+
+    eq_or_diff $have, $want, 'Unicode, baby';
+
+
+Here's what you get:
+
+    #   Failed test 'Unicode, baby'
+    #   at t/unicode.t line 12.
+    # +----+-----------------------+-----------------------+
+    # | Elt|Got                    |Expected               |
+    # +----+-----------------------+-----------------------+
+    # |   0|'Traditional Chinese'  |'Traditional Chinese'  |
+    # *   1|'\x{4e2d}\x{56fd}'     |'\x{4e2d}\x{570b}'     *
+    # +----+-----------------------+-----------------------+
+    # Looks like you failed 1 test of 1.
+    Dubious, test returned 1 (wstat 256, 0x100)
+    Failed 1/1 subtests
+
+That's better, but still awful. However, if you have C<Text::Diff> 0.40 or
+higher installed, you can add this to your code:
+
+    BEGIN { $ENV{DIFF_OUTPUT_UNICODE} = 1 }
+
+Make sure you do this I<before> you load L<Text::Diff>. Then this is the output:
+
+    # +----+-----------------------+-----------------------+
+    # | Elt|Got                    |Expected               |
+    # +----+-----------------------+-----------------------+
+    # |   0|'Traditional Chinese'  |'Traditional Chinese'  |
+    # *   1|'中国'                 |'中國'                 *
+    # +----+-----------------------+-----------------------+
+
+=head1 DEPLOYING
 
 There are several basic ways of deploying Test::Differences requiring more or less
 labor by you or your users.
@@ -261,14 +353,15 @@ if you do this.
 
 =cut
 
-$VERSION = '0.49_02';
+our $VERSION = "0.61_01"; # or "0.001_001" for a dev release
+$VERSION = eval $VERSION;
 
 use Exporter;
 
 @ISA    = qw( Exporter );
-@EXPORT = qw( 
-  eq_or_diff 
-  eq_or_diff_text 
+@EXPORT = qw(
+  eq_or_diff
+  eq_or_diff_text
   eq_or_diff_data
   unified_diff
   context_diff
@@ -339,7 +432,7 @@ sub _flatten {
     my $type = shift;
     local $_ = shift if @_;
 
-    return [ split /^/m ] unless ref;
+    return [ split /^/m, _quote_str($_) ] unless ref;
 
     croak "Can't flatten $_" unless $type;
 
@@ -354,7 +447,10 @@ sub _flatten {
     else {
         die "unsupported ref type";
     }
-    if ( $type eq ARRAY_of_ARRAYs_of_scalars ) {
+    if ( $type eq ARRAY_of_scalars) {
+        @recs = map { _quote_str($_) } @recs;
+    }
+    elsif ( $type eq ARRAY_of_ARRAYs_of_scalars ) {
         ## Also copy the inner arrays if need be
         $_ = [@$_] for @recs;
     }
@@ -381,10 +477,10 @@ sub _flatten {
     }
 
     if ( $type eq ARRAY_of_ARRAYs_of_scalars ) {
-        ## Convert undefs
+        ## Quote strings
         for my $rec (@recs) {
             for (@$rec) {
-                $_ = "<undef>" unless defined;
+                $_ = _quote_str($_);
             }
             $rec = join ",", @$rec;
         }
@@ -393,6 +489,14 @@ sub _flatten {
     return \@recs;
 }
 
+sub _quote_str {
+    my $str = shift;
+    return 'undef' unless defined $str;
+    return $str if $str =~ /^[0-9]+$/;
+    $str =~ s{([\\\'])}{\\$1}g;
+    return "'$str'";
+}
+
 sub _identify_callers_test_package_of_choice {
     ## This is called at each test in case Test::Differences was used before
     ## the base testing modules.
@@ -427,11 +531,18 @@ sub eq_or_diff {
     $options = pop if @_ > 2 && ref $_[-1];
     ( $vals[0], $vals[1], $name ) = @_;
 
-    my $data_type;
-    $data_type = $options->{data_type} if $options;
+    my($data_type, $filename_a, $filename_b);
+    if($options) {
+        $data_type  = $options->{data_type};
+        $filename_a = $options->{filename_a};
+        $filename_b = $options->{filename_b};
+    }
     $data_type ||= "text" unless ref $vals[0] || ref $vals[1];
     $data_type ||= "data";
 
+    $filename_a ||= 'Got';
+    $filename_b ||= 'Expected';
+
     my @widths;
 
     my @types = map _grok_type, @vals;
@@ -442,11 +553,12 @@ sub eq_or_diff {
     if ($dump_it) {
         require Data::Dumper;
         local $Data::Dumper::Indent    = 1;
-        local $Data::Dumper::Sortkeys  = 1;
         local $Data::Dumper::Purity    = 0;
         local $Data::Dumper::Terse     = 1;
         local $Data::Dumper::Deepcopy  = 1;
         local $Data::Dumper::Quotekeys = 0;
+        local $Data::Dumper::Sortkeys =
+          exists $options->{Sortkeys} ? $options->{Sortkeys} : 1;
         ( $got, $expected ) = map
           [ split /^/, Data::Dumper::Dumper($_) ],
           @vals;
@@ -479,8 +591,8 @@ sub eq_or_diff {
         $diff = diff $got, $expected,
           { CONTEXT     => $context,
             STYLE       => _diff_style(),
-            FILENAME_A  => "Got",
-            FILENAME_B  => "Expected",
+            FILENAME_A  => $filename_a,
+            FILENAME_B  => $filename_b,
             OFFSET_A    => $data_type eq "text" ? 1 : 0,
             OFFSET_B    => $data_type eq "text" ? 1 : 0,
             INDEX_LABEL => $data_type eq "text" ? "Ln" : "Elt",
@@ -568,17 +680,17 @@ while.  Note that the two hashes should report the same here:
 
     not ok 5
     #     Failed test (t/ctrl/05-home.t at line 51)
-    # +----+------------------------+----+------------------------+   
-    # | Elt|Got                     | Elt|Expected                |   
-    # +----+------------------------+----+------------------------+   
-    # |   0|{                       |   0|{                       |   
-    # |   1|  'password' => '',     |   1|  'password' => '',     |   
-    # *   2|  'method' => 'login',  *    |                        |   
-    # |   3|  'ctrl' => 'home',     |   2|  'ctrl' => 'home',     |   
-    # |    |                        *   3|  'method' => 'login',  *   
-    # |   4|  'email' => 'test'     |   4|  'email' => 'test'     |   
-    # |   5|}                       |   5|}                       |   
-    # +----+------------------------+----+------------------------+   
+    # +----+------------------------+----+------------------------+
+    # | Elt|Got                     | Elt|Expected                |
+    # +----+------------------------+----+------------------------+
+    # |   0|{                       |   0|{                       |
+    # |   1|  'password' => '',     |   1|  'password' => '',     |
+    # *   2|  'method' => 'login',  *    |                        |
+    # |   3|  'ctrl' => 'home',     |   2|  'ctrl' => 'home',     |
+    # |    |                        *   3|  'method' => 'login',  *
+    # |   4|  'email' => 'test'     |   4|  'email' => 'test'     |
+    # |   5|}                       |   5|}                       |
+    # +----+------------------------+----+------------------------+
 
 Data::Dumper also overlooks the difference between
 
@@ -597,14 +709,14 @@ Yves Orton <demerphq@hotmail.com>.  The plan to address this is to allow
 you to select Data::Denter or some other module of your choice as an
 option.
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-    Barrie Slaymaker <barries@slaysys.com>
-
-=head1 MAINTAINER
+    Barrie Slaymaker <barries@slaysys.com> - original author
 
     Curtis "Ovid" Poe <ovid@cpan.org>
 
+    David Cantrell <david@cantrell.org.uk>
+
 =head1 LICENSE
 
 Copyright 2001-2008 Barrie Slaymaker, All Rights Reserved.
